<?php
/**
 * Plugin Name: Ray's Login Widget
 * Plugin URI: http://www.raymercer.net
 * Description: A simple widget to replace the default Wordpress "meta" widget
 * Version: 0.2 - added custom logo to login page (props to Mark Jaquith for Login Logo plugin)
 * Create a PNG file named login-logo.png which is 400px wide or smaller (transparent works best) upload it to the wp-content dir and the plugin will use it on the admin login page
 * Version: 0.1 - removed all the extra links from the default meta widget so there is simply a login link
 * 
 * License: GPL
 * Author: Ray Mercer
 * Author URI: http://www.raymercer.net
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
 * Add function to widgets_init that'll load our widget.
 * @since 0.1
 */
add_action( 'widgets_init', 'rays_load_widgets' );

/**
 * Register our widget.
 * 'Rays_Login_Widget' is the widget class used below.
 *
 * @since 0.1
 */
function rays_load_widgets() {
	register_widget( 'Rays_Login_Widget' );
}

/**
 * Widget class.
 * This class handles everything that needs to be handled with the widget:
 * the settings, form, display, and update.  Nice!
 *
 * @since 0.1
 */
class Rays_Login_Widget extends WP_Widget {
    static $instance;
	const cutoff = 400;
	var $logo_locations;
	var $logo_location;
	var $width = 0;
	var $height = 0;
	var $original_height;
	var $logo_size;
	var $logo_file_exists;

	/**
	 * Widget setup.
	 */
	public function Rays_Login_Widget() {
		self::$instance = $this;
		add_action( 'login_head', array( $this, 'login_head' ) );
	
		/* Widget settings. */
		$widget_ops = array( 'classname' => 'rays_login', 'description' => __('A simple widget to replace the default Wordpress meta widget', 'rays_login') );

		/* Create the widget. */
		$this->WP_Widget( 'rays-login-widget', __('Rays Login Widget', 'rays_login'), $widget_ops, $control_ops );
	}

	public function init() {
			$this->logo_locations = array();
			if ( is_multisite() && function_exists( 'get_current_site' ) ) {
				$site = get_current_site(); // Site = Network? Ugh.
				if ( $site && isset( $site->id ) ) {
					$this->logo_locations['network'] = array(
						'path' => WP_CONTENT_DIR . '/login-logo-network-' . $site->id . '.png',
						'url' => $this->maybe_ssl( WP_CONTENT_URL . '/login-logo-network-' . $site->id . '.png' )
						);
				}
			}
			$this->logo_locations['global'] =  array(
				'path' => WP_CONTENT_DIR . '/login-logo.png',
				'url' => $this->maybe_ssl( WP_CONTENT_URL . '/login-logo.png' )
				);
		}


	private function maybe_ssl( $url ) {
			if ( is_ssl() )
				$url = preg_replace( '#^http://#', 'https://', $url );
			return $url;
		}

	private function logo_file_exists() {
			if ( ! isset( $this->logo_file_exists ) ) {
				foreach ( $this->logo_locations as $location ) {
					if ( file_exists( $location['path'] ) ) {
						$this->logo_file_exists = true;
						$this->logo_location = $location;
						break;
					} else {
						$this->logo_file_exists = false;
					}
				}
			}
			return !! $this->logo_file_exists;
		}

	private function get_location( $what = '' ) {
			if ( $this->logo_file_exists() ) {
				if ( 'path' == $what || 'url' == $what )
					return $this->logo_location[$what];
				else
					return $this->logo_location;
			}
			return false;
		}

	private function get_width() {
			$this->get_logo_size();
			return absint( $this->width );
		}

	private function get_height() {
			$this->get_logo_size();
			return absint( $this->height );
		}

	private function get_original_height() {
			$this->get_logo_size();
			return absint( $this->original_height );
		}

	private function get_logo_size() {
			if ( !$this->logo_file_exists() )
				return false;
			if ( !$this->logo_size ) {
				if ( $sizes = getimagesize( $this->get_location( 'path' ) ) ) {
					$this->logo_size = $sizes;
					$this->width  = $sizes[0];
					$this->height = $sizes[1];
					$this->original_height = $this->height;
					if ( $this->width > self::cutoff ) {
						// Use CSS 3 scaling
						$ratio = $this->height / $this->width;
						$this->height = ceil( $ratio * self::cutoff );
						$this->width = self::cutoff;
					}
				} else {
					$this->logo_file_exists = false;
				}
			}
			return array( $this->width, $this->height );
		}

	private function css3( $rule, $value ) {
			foreach ( array( '', '-o-', '-webkit-', '-khtml-', '-moz-', '-ms-' ) as $prefix ) {
				echo $prefix . $rule . ': ' . $value . '; ';
			}
		}

	public function login_headerurl() {
			return trailingslashit( get_bloginfo( 'url' ) );
		}

	public function login_head() {
			$this->init();
			if ( !$this->logo_file_exists() )
				return;
			add_filter( 'login_headerurl', array( $this, 'login_headerurl' ) );
		?>
		<style type="text/css">
			h1 a {
				background: url(<?php echo esc_url_raw( $this->get_location( 'url' ) ); ?>) no-repeat top center;
				width: <?php echo self::cutoff; ?>px;
				height: <?php echo $this->get_height() + 3; ?>px;
				<?php if ( self::cutoff < $this->get_width() ) $this->css3( 'background-size', 'contain' ); ?>
			}
		</style>
	<?php if ( self::cutoff < $this->get_width() ) { ?>
	<!--[if lt IE 9]>
		<style type="text/css">
			height: <?php echo $this->get_original_height() + 3; ?>px;
		</style>
	<![endif]-->
	<?php
			}
	}



	/**
	 * How to display the widget on the screen.
	 */
	function widget( $args, $instance ) {
		extract( $args );

		/* Our variables from the widget settings. */
		$title = apply_filters('widget_title', $instance['title'] ) ;

		/* Before widget (defined by themes). */
		echo $before_widget;

		/* Display the widget title if one was input (before and after defined by themes). */
		if ( $title )
			echo $before_title . $title . $after_title;
?>
			<ul>
			<?php wp_register(); ?>
			<li><?php wp_loginout(); ?></li>
			<?php wp_meta(); ?>
			</ul>
<?php



		/* After widget (defined by themes). */
		echo $after_widget;
	}

	/**
	 * Update the widget settings.
	 */
	function update( $new_instance, $old_instance ) {
		$instance = $old_instance;

		/* Strip tags for title and name to remove HTML (important for text inputs). */
		$instance['title'] = strip_tags( $new_instance['title'] );

		return $instance;
	}

	/**
	 * Displays the widget settings controls on the widget panel.
	 * Make use of the get_field_id() and get_field_name() function
	 * when creating your form elements. This handles the confusing stuff.
	 */
	function form( $instance ) {

		$instance = wp_parse_args( (array) $instance, array( 'title' => '' ) );
		$title = strip_tags($instance['title']);
?>
		<!-- Widget Title: Ray's Login Widget -->

		<p>
			<label for="<?php echo $this->get_field_id( 'title' ); ?>"><?php _e('Title:', 'hybrid'); ?></label>
			<input id="<?php echo $this->get_field_id( 'title' ); ?>" name="<?php echo $this->get_field_name( 'title' ); ?>" value="<?php echo $instance['title']; ?>" style="width:100%;" />
		</p>

	<?php
	}
}

?>